<?php

namespace JimsCustomerBooking\Database;

use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Helpers\TimeHelper;
use JimsCustomerBooking\Services\DivisionService;

/**
 * Class Booking
 *
 * @package JimsCustomerBooking\Database
 */
class Booking extends Model
{
    protected $table_name = 'customer_bookings';

    /**
     * Returns the ID of the local booking
     *
     * @param $data
     * @param $status
     * @return int
     */
    public function add($data, $status = 'pending', $error_code = null)
    {
        $title = $this->calculateTitleFromData($data);

        if(self::thereIsTheSameToday(json_encode($data))){
            return 0;
        }

        return $this->insert([
            'title' => $title,
            'booking_id' => md5(time() . $title),
            'data' => \json_encode($data),
            'status' => $status,
            'error_code' => $error_code,
            'created_at' => date('Y-m-d H:i:s'),
            'updated_at' => date('Y-m-d H:i:s'),
        ]);
    }

    protected function calculateTitleFromData($data)
    {
        $divisionService = new DivisionService();
        $services = $divisionService->getServices(true);
        $first_service = $data['services'][0];

        $service = array_filter($services, function ($item) use ($first_service) {
            return $item['id'] == $first_service;
        });

        $service = array_shift($service);

        return $service['name'];
    }

    /**
     * @param $page
     * @param $limit
     * @param $status
     * @return array|null
     */
    public function getPage($page = 1, $limit = 25, $status = 'all')
    {
        $offset = ($page - 1) * $limit;
        $table = $this->getTable($this->table_name);

        if ($status == 'all') {
            $sql = "SELECT * FROM $table ORDER BY created_at ASC LIMIT %d OFFSET %d";
            $sql = $this->prepare($sql, [$limit, $offset]);
        } else {
            $sql = "SELECT * FROM $table WHERE booking_status = %s ORDER BY created_at ASC LIMIT %d OFFSET %d";
            $sql = $this->prepare($sql, [$status, $limit, $offset]);
        }

        //apply filter
        $sql = apply_filters(StringHelper::GetPrefixed('query_booking_get_page'), $sql, $page, $limit, $status);

        return $this->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    /**
     * @param $limit
     * @param $status
     * @return int
     */
    public static function getPageCount($limit = 10, $status = 'all')
    {
        $self = new static;
        $table = $self->getTable();

        if ($status == 'all') {
            $sql = "SELECT COUNT(*) FROM $table";
        } else {
            $sql = "SELECT COUNT(*) FROM $table WHERE status = %s";
            $sql = $self->prepare($sql, [$status]);
        }

        //apply filter
        $sql = apply_filters(StringHelper::GetPrefixed('query_booking_get_page_count'), $sql, $limit, $status);

        $all_records_count = $self->get_var($sql);
        $pages = ceil($all_records_count / $limit);

        return $pages;
    }

    /**
     * @param $status
     * @return array|object|\stdClass[]|null
     */
    public static function getStatisticsTotals($status = 'sended', $start_date = null, $end_date = null, $timezone = '')
    {
        $self = new static;
        $table = $self->getTable();

        if( empty($start_date) ) {
            $sql = "SELECT title, COUNT(*) AS total_bookings, SUM(CASE WHEN status = %s THEN 1 ELSE 0 END) AS total_bookings_with_status FROM $table GROUP BY title ORDER BY title, created_at;";
            $sql = $self->prepare($sql, [$status]);
        } else {

            //check if start_date is formatted like a date
            if(preg_match('/^\d{2}\/\d{2}\/\d{4}$/', $start_date) === 0){
                return [];
            }

            if (empty($end_date)) {
                $end_date = date('d/m/Y');
            }

            $sql = "SELECT title, COUNT(*) AS total_bookings, SUM(CASE WHEN status = '$status' THEN 1 ELSE 0 END) AS total_bookings_with_status FROM $table WHERE DATE(created_at) BETWEEN STR_TO_DATE('$start_date', '%d/%m/%Y') AND STR_TO_DATE('$end_date', '%d/%m/%Y') GROUP BY title ORDER BY title, created_at;";
        }

        return $self->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    /**
     * @param $date
     * @param $status
     * @return array|object|\stdClass[]|null
     */
    public static function getStatisticsTotalsByDate($date = null, $status = 'sended')
    {
        $self = new static;
        $table = $self->getTable();

        if ($date === null) {
            $date = date('d/m/Y');
        }

        $sql = "SELECT title, COUNT(*) AS total_bookings, SUM(CASE WHEN status = '$status' THEN 1 ELSE 0 END) AS total_bookings_with_status FROM $table WHERE DATE(created_at) = STR_TO_DATE('$date', '%d/%m/%Y') GROUP BY title ORDER BY title, created_at;";
//        $sql = $self->prepare($sql, [$status, $date]);

        return $self->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    /**
     * @param $offset
     * @param $limit
     * @return array|object|\stdClass[]|null
     */
    public static function getStatisticsAll($offset = 0, $limit = 10)
    {
        $self = new static;
        $table = $self->getTable();

        $sql = "SELECT * FROM $table ORDER BY created_at ASC LIMIT %d OFFSET %d";
        $sql = $self->prepare($sql, [$limit, $offset]);

        return $self->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    /**
     * @param $status
     * @param $start_date
     * @param $end_date
     * @param $offset
     * @param $limit
     * @return array|object|\stdClass[]|null
     */
    public static function getStatisticsAllByPeriod($status = 'sended', $start_date = null, $end_date = null, $offset = 0, $limit = 10, $timezone = '+11:00')
    {
        $self = new static;
        $table = $self->getTable();

        if ($start_date === null) {
            $start_date = date('d/m/Y');
        }

        if ($end_date === null) {
            $end_date = date('d/m/Y');
        }

        if( empty($timezone) || $timezone === '+00:00' ){
            $sql = "SELECT * FROM $table WHERE DATE(created_at)  BETWEEN STR_TO_DATE('$start_date', '%d/%m/%Y') AND STR_TO_DATE('$end_date', '%d/%m/%Y') ORDER BY created_at ASC LIMIT $limit OFFSET $offset";
        }else{
            $sql = "SELECT * FROM $table WHERE CONVERT_TZ(created_at, '+00:00', '$timezone') BETWEEN STR_TO_DATE('$start_date', '%d/%m/%Y') AND STR_TO_DATE('$end_date', '%d/%m/%Y') ORDER BY created_at ASC LIMIT $limit OFFSET $offset";
        }

        return $self->get_results($sql, self::RESULT_TYPE_ARRAY_A);
    }

    /**
     * @param $data
     * @return bool
     */
    public static function thereIsTheSameToday($data){
        $self = new static;
        $table = $self->getTable();

        $sql = "SELECT * FROM $table WHERE DATE(created_at) = CURDATE() AND data = %s";
        $sql = $self->prepare($sql, [$data]);

        return count($self->get_results($sql, self::RESULT_TYPE_ARRAY_A)) > 0;
    }

    /**
     * @param $status
     * @param $start_date
     * @param $end_date
     * @return mixed
     */
    public static function getStatisticsPeriodTotal($status = 'sended', $start_date = null, $end_date = null){
        $self = new static;
        $table = $self->getTable();

        if ($start_date === null) {
            $start_date = date('d/m/Y');
        }

        if ($end_date === null) {
            $end_date = date('d/m/Y');
        }

        $sql = "SELECT COUNT(*) AS total_bookings, SUM(CASE WHEN status = '$status' THEN 1 ELSE 0 END) AS total_bookings_with_status, (SUM(CASE WHEN status = '$status' THEN 1 ELSE 0 END) / COUNT(*)) as rate FROM $table WHERE DATE(created_at) BETWEEN STR_TO_DATE('$start_date', '%d/%m/%Y') AND STR_TO_DATE('$end_date', '%d/%m/%Y')";

        return $self->get_row($sql, self::RESULT_TYPE_ARRAY_A);
    }
}