<?php

namespace JimsCustomerBooking\Integrations;

use JimsCustomerBooking\BookingForm;
use JimsCustomerBooking\Helpers\Http;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsGroup\Apis\CustomerBooking;
use JimsGroup\Datatypes\AddressType;
use JimsGroup\Datatypes\BookingResponseType;
use mysql_xdevapi\Exception;

/**
 * Class JimsFMS
 *
 * @package JimsCustomerBooking\Integrations
 * This class is used to integrate with JimsFMS
 */
class JimsFmsBooking
{
    /**
     * @var CustomerBooking $customer_booking
     */
    private $customer_booking = null;


    /**
     * It selects the server url
     *
     * @var string $mode
     */
    private $mode = 'production'; // 'production', 'uat', 'hf', 'hf2', 'development


    private $responses = [];

    private $messages = [];

    /**
     * @var BookingForm $booking_form
     */
    private $booking_form = null;

    /**
     * @var string $api_key
     */
    private $api_key = '3e8bce9e-be45-4499-b31a-23b6fb067aa7';

    public $errors = [];

    public function __construct($countryCode = 'AU', $mode = 'production')
    {
        // set mode
        $this->mode = $mode;

        // initialize customer booking api
        $this->customer_booking = new CustomerBooking();

        // set mode and api key, it selects the server url
        $this->customer_booking->setMode($this->mode);

        // set api key
        $this->customer_booking->setAuthToken($this->api_key);


        try {
            // get initial booking request
            $this->doInitialRequest($countryCode);
        } catch (\Exception $e) {
            $this->errors[] = $e->getMessage();
        }

    }

    public function getApiUrl()
    {
        return $this->customer_booking->getApiUrl();
    }

    /**
     * @param $countryCode
     * @return mixed
     */
    public function doInitialRequest($countryCode = 'AU')
    {
        // get transient name
        $transientName = StringHelper::GetPrefixedFromParts(['fms_initial_request', $countryCode]);

        $requestResponse = get_transient($transientName);

        // get transient
        if ( false === $requestResponse ) {
            $requestResponse = [];

            try {
                // get initial booking request
                $requestResponse = $this->customer_booking->getInitialBookingRequest([
                    'countryCode' => $countryCode,
                ]);

                // set transient
                set_transient($transientName, $requestResponse, 1 * HOUR_IN_SECONDS);
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            }
        }

        return $requestResponse;
    }

    public function doBookingRequest($customer, $address, $division_id, array $service_ids)
    {
        //Store the data to a BookingForm object
        $this->booking_form = new BookingForm($customer, $address, $division_id, $service_ids);

        error_log(json_encode($this->booking_form->toArray()), 0);

        //try direct booking or allocation
        $booking = false;
        try {
            $booking = $this->tryToBook($this->booking_form);
        } catch (Exception $e) {
            error_log(implode(' ', [$e->getFile(), $e->getLine(), $e->getCode(), $e->getMessage()]), 0);
        }

        return $booking;
    }

    /**
     * @param BookingForm $form
     * @param $methods
     * @return bool
     */
    public function tryToBook(BookingForm $form, $methods = ['doAllocationRequest', 'doFinalRequest'])
    {
        foreach ($methods as $method) {
            /**
             * @var BookingResponseType $response
             */
            $response = $this->$method ($form);
            $this->addResponse($response);

            error_log(__FILE__ . ' ' . json_encode($response->toArray()), 0);

            //add check for message and status codes here
            $this->setMessage(is_string($response) ? $response : json_encode($response->toArray()));

            $status = json_encode(['message' => $response->getStatusMessage(), 'code' => $response->getConfirmCodes()]);

            if ($response->isBookingFailed()) {
                $this->setMessage($status);
                throw new \Exception ($response->getStatusMessage());
            } elseif ($response->isBookingCompleted()) {
                $this->setMessage($status);
                return true;
            }
        }

        return false;
    }

    /**
     * @param BookingForm $form
     * @return BookingResponseType
     */
    public function doAllocationRequest(BookingForm $form)
    {
        $address = new AddressType($form->getAddress());

        return $this->customer_booking->sendAllocationRequest(
            $form->getDivisionId(),
            $form->getServices(),
            $address->toArray(),
            $form->getCustomerInfo(),
            $this->customer_booking->getAuthToken(),
        );
    }

    /**
     * @param BookingForm $form
     * @return BookingResponseType
     */
    public function doFinalRequest(BookingForm $form, $previous_response = null, $used_jims_before = false)
    {
        if ($previous_response === null) {
            $response = $this->getLastResponse();

            if ($response->result === false) {
                return new BookingResponseType(json_encode([
                    'result' => false,
                    'message' => $response->message,
                    'status' => 'failed',
                    'confirmCodes' => $response->confirmCodes
                ]));
            }
        } else {
            $response = $previous_response;
        }

        //get properties from the last response, that is not standard named
        list($booking_request_id, $provider, $provider_id) = $this->getUnsafeProperties($response);

        error_log('Used Jims Before: ' . $used_jims_before, 0);

        //get the last response and send the final request
        $params = [
            'booking_request_id' => $booking_request_id,
            'division_id' => $form->getDivisionId(),
            'service_ids' => $form->getServices(),
            'choosedProviderId' => $provider_id,
            'user_ip_address' => Http::getUserIP(),
            'provider_info' => $provider,
            'user_info' => $form->getCustomerInfo(),
            'bookingConfirmCode' => $response->getPayloadProperty('confirmCodes'),
            'previous_lead_id' => $used_jims_before ? $response->getPreviousLeadId() : 0,
        ];

        error_log(json_encode($params), 0);

        //$booking_request_id, $division_id, $service_ids, $choosedProviderId, $user_ip_address, $provider_info, $user_info, $bookingConfirmCode
        return $this->customer_booking->sendFinalizeBookingRequest(...$params);
    }

    /**
     * @param $division_id
     * @return \JimsGroup\Datatypes\Collection|mixed
     */
    public function getDivisionServices($division_id)
    {
        $transientName = StringHelper::GetPrefixed('fms_division_services_' . $division_id);
        if (false === ($requestResponse = get_transient($transientName))) {
            $requestResponse = $this->customer_booking->getDivisionServices($division_id);
            set_transient($transientName, $requestResponse, 12 * HOUR_IN_SECONDS);
        }
        return $requestResponse;
    }

    /**
     * @return array|\JimsGroup\Datatypes\Collection|mixed
     */
    public function getDivisions()
    {
        $transientName = StringHelper::GetPrefixed('fms_divisions');
        if (false === ($requestResponse = get_transient($transientName))) {
            $requestResponse = $this->customer_booking->getDivisions();
            set_transient($transientName, $requestResponse, 12 * HOUR_IN_SECONDS);
        }
        return $requestResponse;
    }

    /**
     * @return array
     */
    public function getMessages(): array
    {
        return $this->messages;
    }

    /**
     * @param array $messages
     */
    public function setMessage($message): void
    {
        $this->messages[] = $message;
    }

    private function addResponse($request)
    {
        $this->responses[] = $request;
    }

    public function getResponses()
    {
        return $this->responses;
    }

    /**
     * @return BookingResponseType
     */
    public function getLastResponse()
    {
        return end($this->responses);
    }

    /**
     * @param BookingResponseType $last_response
     * @return array
     */
    protected function getUnsafeProperties(BookingResponseType $last_response): array
    {
//get bookingRequestId or booking_request_id
        $booking_request_id = $last_response->getPayloadProperty('booking_request_id');
        if ($booking_request_id === null) {
            $booking_request_id = $last_response->getPayloadProperty('bookingRequestId');
        }

        //get first available fse
        $avaliableFSEs = $last_response->getPayloadProperty('avaliableFSEs');
        $provider = $avaliableFSEs[0];

        //get ProviderId
        $provider_id = $last_response->getPayloadProperty('provider_id');
        if (empty($provider_id)) {
            $provider_id = $provider['business_id'];
        }
        return array($booking_request_id, $provider, $provider_id);
    }
}